# -*- coding: UTF-8 -*-

# Phatch - Photo Batch Processor
# Copyright (C) 2007-2008  www.stani.be
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/

__all__ = ['main']

import sys
if sys.version_info[0] != 2:
    sys.exit('Sorry, Phatch is only compatible with Python 2.x!\n')

import logging
import os
import optparse
import shutil
import subprocess

from lib import i18n
from lib import system

from lib.context import join
from core.context import CONTEXT, check_user_paths

if CONTEXT['windows']:
    from lib.windows import sysArgvUnicode
    sysArgvUnicode.fix()

VERSION = "%(app_name)s %(version_app)s" % CONTEXT


def set_font_cache():
    """Only sets font cache global variables, does not execute anything."""
    from lib.fonts import set_font_cache
    set_font_cache(
        CONTEXT['user_fonts_path'],
        CONTEXT['app_fonts_path'],
        CONTEXT['user_fonts_cache_path'],
        CONTEXT['app_fonts_cache_path']
    )


def apply_pil_patches():
    """Patches for pil <= 1.1.6"""
    import Image
    if Image.VERSION < '1.1.7':
        sys.path.insert(0,
            join(CONTEXT['app_source_path'], 'other', 'pil_1_1_6'))


def parse_options():

    parser = optparse.OptionParser(
        usage="""
  %(app_name)s [actionlist]
  %(app_name)s [options] [actionlist] [image folders/files/urls]
  %(app_name)s --inspect [image files/urls]
  %(app_name)s --droplet [actionlist/recent] [image files/urls]""" % CONTEXT \
    + """

%s:
  phatch action_list.phatch
  phatch --verbose --recursive action_list.phatch image_file.png image_folder
  phatch --inspect image_file.jpg
  phatch --droplet recent""" % _('Examples'),
        version=VERSION,
    )
    # Options in alphabetical order
    # If you add an option, also add it to the man page!
    parser.add_option("-c", "--console", action="store_true",
        dest="console",
        default=False,
        help=_("Run %s as console program without a gui") \
            % CONTEXT['app_title'])
    parser.add_option("--context", action="store_true",
        dest="list_context",
        default=False,
        help=_("Show application and system configuration"))
    parser.add_option("-d", "--droplet", action="store_true",
        dest="droplet",
        default=False,
        help=_("Run %s as a gui droplet") % CONTEXT['app_name'])
    parser.add_option("--desktop", action="store_true",
        dest="desktop",
        default=False,
        help=_("Always save on desktop"))
    parser.add_option("-f", "--force", action="store_false",
        dest="stop_for_errors",
        default=True,
        help=_("Ignore errors"))
    parser.add_option("--fonts", action="store_true",
        dest="init_fonts",
        default=False,
        help=_("Initialize fonts (only for installation scripts)"))
    parser.add_option("-i", "--interactive", action="store_true",
        dest="interactive",
        default=False,
        help=_("Interactive"))
    parser.add_option("-k", "--keep", action="store_false",
        dest="overwrite_existing_images",
        default=True,
        help=_("Keep existing images (don't overwrite)"))
    parser.add_option("-l", action="store",
        dest="locale",
        default='default',
        type="string",
        help=_("Specify locale language (for example en or en_GB)"))
    parser.add_option("--log", action="store",
        dest="log_level",
        type='int',
        default=logging.WARN,
        help=_("Logging level"))
    parser.add_option("-n", "--inspect", action="store_true",
        dest="image_inspector",
        default=False,
        help=_("Inspect metadata (requires exif & iptc plugin)"))
    parser.add_option("--no-save", action="store_true",
        dest="no_save",
        default=False,
        help=_("No save action required at the end"))
    parser.add_option("-r", "--recursive", action="store_true",
        dest="recursive",
        default=False,
        help=_("Include all subfolders"))
    parser.add_option("-t", "--trust", action="store_false",
        dest="check_images_first",
        default=True,
        help=_("Do not check images first"))
    parser.add_option("--unsafe", action="store_false",
        dest="safe",
        default=True,
        help=_("Allow Geek action and unsafe expressions"))
    parser.add_option("-v", "--verbose", action="store_true",
        dest="verbose",
        default=False,
        help=_("Verbose"))
    options, paths = parser.parse_args()
    paths = [system.fix_path(path) for path in paths
        if path and path[0] != '%']
    return options, paths


def load_locale():
    if '-l' in sys.argv:
        index = sys.argv.index('-l') + 1
        if index >= len(sys.argv):
            sys.exit('Please specify locale language after "-l".')
        canonical = sys.argv[index]
    else:
        canonical = 'default'
    i18n.load_locale('phatch', CONTEXT["app_locale_path"], canonical)


PYWX_ERROR = """\
Only the command line package 'phatch-cli' seems to be installed.
Please install the graphical user interface package 'phatch' as well.
"""


def get_gui(exit=True):
    from pyWx import gui
    try:
        from pyWx import gui
        return gui
    except ImportError:
        if exit:
            sys.exit(PYWX_ERROR)


def init_fonts():
    from lib.fonts import font_dictionary
    font_dictionary(force=True)


def init_log(level):
    logger = logging.getLogger()
    logger.setLevel(level)


def show_context():
    import pprint
    from core.context import CONTEXT
    get_gui(exit=False)
    pprint.pprint(CONTEXT)


def start_gui(paths, settings):
    system.reexec_with_pythonw()  # ensure pythonw for mac
    if paths:
        actionlist = paths[0]
    else:
        actionlist = ''
    get_gui().main(settings, actionlist)


def start_inspector(paths):
    system.reexec_with_pythonw()  # ensure pythonw for mac
    get_gui().inspect(paths)


def start_droplet(paths, settings):
    system.reexec_with_pythonw()  # ensure pythonw for mac
    get_gui().drop(actionlist=paths[0], paths=paths[1:], settings=settings)


def start_console(paths, settings):
    from core.api import init
    init()
    from console import console
    if paths and system.has_ext(paths[0], CONTEXT['app_extension']):
        console.main(actionlist=paths[0], paths=paths[1:], settings=settings)
    else:
        console.main(actionlist='', paths=paths, settings=settings)


def create_settings(options=None):
    settings = {
        #execute
        'extensions': [],
        'recursive': False,
        'stop_for_errors': True,
        'overwrite_existing_images': True,
        'no_save': False,
        'check_images_first': True,
        'always_show_status_dialog': True,
        "desktop": False,
        "safe": True,
        "repeat": 1,
        #console
        'console': False,
        'init_fonts': False,
        'interactive': False,
        'verbose': True,
        #gui
        'browse_source': 0,
        'tag_actions': _('All'),
        'description': True,
        'collapse_automatic': False,
        'droplet': False,
        'droplet_path': CONTEXT['user_path'],
        'file_history': [],
        'image_inspector': False,
        'paths': [CONTEXT['user_path'], ],
        #internal
        'overwrite_existing_images_forced': False,
    }
    if options:
        for attr in settings:
            if hasattr(options, attr):
                settings[attr] = getattr(options, attr)
    return settings


def main():
    """init should be called first!"""
    # process settings
    load_locale()
    options, paths = parse_options()
    settings = create_settings(options)
    init_log(options.log_level)
    if options.list_context:
        show_context()
        return
    if settings['verbose']:
        system.VERBOSE = True
    # safe can only be set by the command line or the ui, and is not permanent
    if 'safe' in settings:
        from lib import formField
        formField.set_safe(settings['safe'])
        del settings['safe']
    # initialize font.cache -> can be done at root level
    set_font_cache()
    if settings['init_fonts']:
        init_fonts()
        return
    # apply pil patches
    apply_pil_patches()
    # start inspector
    if settings['image_inspector']:
        start_inspector(paths)
        return
    # check for font.cache, geek.txt
    check_user_paths()
    # force to choose from recent action lists if no action list is given
    if paths and not (paths[0] == 'recent' or \
            system.has_ext(paths[0], CONTEXT['app_extension'])):
        settings['droplet'] = True
        paths.insert(0, 'recent')
    # start droplet, console or gui
    if settings['droplet']:
        if not paths:
            paths = ['recent']
        start_droplet(paths, settings)
    elif len(paths) > 1 or settings['console'] or settings['interactive']:
        start_console(paths, settings)
    else:
        start_gui(paths, settings)

if __name__ == '__main__':
    main()
