# Copyright (C) 2007-2010 www.stani.be
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/

__all__ = ['BIN', 'get_context', 'join', 'join_create', 'ContextError']

import time
CONTEXT = {'start_time': time.time()}

import os
import sys
import platform

import desktop


def endswith(x, suffices):
    for suffix in suffices:
        if x.endswith(suffix):
            return True
    return False


def contains(x, els):
    for el in els:
        if el in x:
            return True
    return False

BIN = False  # should be set to True by /usr/bin/app
PATHS = '--paths' in sys.argv
SETUP_PY = sys.argv[0].endswith('setup.py')
SYS_PREFIX_LOCAL = os.path.join(sys.prefix, 'local')
FREEZING = SETUP_PY and contains(sys.argv, ('py2app', 'py2exe'))


# platform
if sys.platform.startswith('win'):
    CONTEXT['platform'] = 'windows'

elif sys.platform.startswith('darwin'):
    CONTEXT['platform'] = 'apple'
else:
    #assume linux
    CONTEXT['platform'] = 'linux'

CONTEXT['windows'] = CONTEXT['platform'] == 'windows'
CONTEXT['apple'] = CONTEXT['platform'] == 'apple'
CONTEXT['linux'] = CONTEXT['platform'] == 'linux'
CONTEXT['unix'] = CONTEXT['platform'] in ('linux', 'apple')

# user
CONTEXT['desktop_path'] = desktop.DESKTOP_PATH
CONTEXT['user_path'] = desktop.USER_PATH

# platform
CONTEXT['version_platform'] = platform.platform()
CONTEXT['version_python'] = sys.version.replace('\n', '')


class ContextError(Exception):
    pass


def join(*args):
    path = os.path.join(*args)
    if args[0] != '':
        path = os.path.abspath(path)
        if BIN and not(PATHS or os.path.exists(path)):
            raise ContextError((path, sys.argv[0]))
    return path


def join_create(*args):
    path = os.path.abspath(os.path.join(*args))
    if not os.path.exists(path):
        os.mkdir(path)
    return path


def _find_parent(path, base):
    if base == os.path.basename(path):
        return path
    if not path:
        raise ContextError('Can not find "%s" parent path.' % base)
    return _find_parent(os.path.abspath(os.path.join(path, '..')), base)


def _find_source_path(base_source):
    # __file__ may only be used to locate the source files!
    # on linux phatch source might be in site-packages or /usr/share/phatch
    # so unfortunately we need to use __file__
    if base_source is None:
        return '?'
    return _find_parent(
        os.path.abspath(
            os.path.dirname(unicode(__file__, sys.getfilesystemencoding())),
        ),
        base_source,
    )


def get_context(app_name, base_source='phatch', sys_prefix=None, user=True):
    """Finds the right paths depending on the context, which consists of the
    platform and how it is distributed.

    context = (platform, distribute)
    platform = 'linux', 'apple' or 'windows'
    distribute = 'frozen', 'package' or 'source'
    """
    if sys_prefix is None:
        sys_prefix = sys.prefix
    context = CONTEXT.copy()
    # initialize APP_NAME
    context['app_name'] = app_name
    context['app_title'] = app_name.title()
    # distribute & context
    if hasattr(sys, 'frozen') or FREEZING:
        context['distribute'] = 'frozen'
    elif SETUP_PY or BIN:
        context['distribute'] = 'package'
    else:
        context['distribute'] = 'source'
    context['context'] = (context['platform'], context['distribute'])
    # initialize data, doc & locale path
    if FREEZING:
        # py2app or py2exe
        context['app_data_path'] = 'data'
        context['app_doc_path'] = 'doc'
        context['app_locale_path'] = 'locale'
    elif context['distribute'] == 'source':
        # source: eg. trunk/phatch
        context['app_source_path'] = _find_source_path(base_source)
        # data: eg. trunk/data
        context['app_data_path'] = join(
            context['app_source_path'], '..', 'data')
        # doc: eg. trunk/doc
        context['app_doc_path'] = join(
            context['app_source_path'], '..', 'doc')
        # locale: eg. trunk/locale
        context['app_locale_path'] = join(
            context['app_source_path'], '..', 'locale')
    elif context['context'] == ('linux', 'package'):
        from linux import path
        # source: eg. /usr/share/phatch/phatch
        # should be retrieved automatically
        context['app_source_path'] = _find_source_path(base_source)
        if context['app_source_path'].startswith(SYS_PREFIX_LOCAL) \
                and sys_prefix == sys.prefix:
            sys_prefix = SYS_PREFIX_LOCAL
        # other paths can be overwritten in lib/linux/path.py
        # data: eg. /usr/share/phatch/data
        if path.DATA is None:
            context['app_data_path'] = join(sys_prefix, 'share',
                app_name, 'data')
        else:
            context['app_data_path'] = path.DATA
        # doc: eg. /usr/share/doc/phatch/
        if path.DOC is None:
            context['app_doc_path'] = join(sys_prefix, 'share', 'doc',
                app_name)
        else:
            context['app_doc_path'] = path.DOC
        # locale: eg. /usr/share/locale/
        if path.LOCALE is None:
            context['app_locale_path'] = join(sys_prefix, 'share', 'locale')
        else:
            context['app_locale_path'] = path.LOCALE
    elif context['context'] == ('apple', 'package'):
        context['app_source_path'] = _find_source_path(base_source)
        context['app_data_path'] = '/Library/Application Support/Phatch'
        context['app_locale_path'] = '/usr/share/locale'
        context['app_doc_path'] = '/usr/share/doc/phatch'
    #elif context['context'] == ('apple', 'frozen'):
    #    # TODO: implement this!
    #    APP_ROOT = os.path.dirname(os.path.dirname(unicode(sys.executable,
    #        sys.getfilesystemencoding())))
    #    # eg. Phatch.app/Contents/Resources/
    #    context['app_data_path'] = join(APP_ROOT, 'Contents', 'Resources')
    elif context['context'] == ('windows', 'frozen'):
        # The same as the exe dir
        context['app_source_path'] = source_path = \
            os.path.dirname(unicode(sys.executable,
                sys.getfilesystemencoding()))
        context['app_data_path'] = os.path.join(source_path, 'data')
        context['app_doc_path'] = os.path.join(source_path, 'doc')
        context['app_locale_path'] = os.path.join(source_path, 'locale')
    else:
        raise ContextError(
            'Sorry your context%(context)s is not yet supported.' % context)

    if not user:
        return context

    # user paths
    app_title = app_name.title()
    if context['windows']:
        if 'APPDATA' in os.environ and os.environ['APPDATA'] \
                and os.path.exists(os.environ['APPDATA']):
            user_path = os.path.join(os.environ['APPDATA'], app_title)
        else:
            user_path = os.path.join(desktop.USER_PATH, app_title)
        join_create(user_path)
        context['user_cache_path'] = join_create(user_path, 'cache')
        context['user_config_path'] = join_create(user_path, 'config')
        context['user_data_path'] = join_create(user_path, 'data')
    elif context['apple']:
        #apple
        context['user_cache_path'] = join_create(desktop.USER_PATH, 'Library',
            'Caches', app_title)
        context['user_config_path'] = join_create(desktop.USER_PATH, 'Library',
            'Preferences', app_title)
        context['user_data_path'] = join_create(desktop.USER_PATH, 'Library',
            'Application Support', app_title)
    else:
        context['user_cache_path'] = join_create(desktop.USER_CACHE_PATH,
            app_name)
        context['user_config_path'] = join_create(desktop.USER_CONFIG_PATH,
            app_name)
        context['user_data_path'] = join_create(desktop.USER_DATA_PATH,
            app_name)

    return context


if __name__ == '__main__':
    import pprint
    pprint.pprint(get_context('app', base_source='phatch'))
