# Copyright (C) 2009 www.stani.be
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/

# Follows PEP8

import os
import re
import shutil
import subprocess
import sys
import types
import tempfile
import textwrap
import urllib

import safe

ARG_STR = '".+?"|\'.+\'|\S+'
BIN_PATHS = []  # executable
FS_ENCODING = sys.getfilesystemencoding()
RE_ARG = re.compile(ARG_STR)
RE_COMMAND = re.compile('^(%s)' % ARG_STR)
RE_NEED_QUOTES = re.compile('^[^\'"].+?\s.+?[^\'"]$')
VERBOSE = False


if sys.platform.startswith('win'):
    _EXE = '.exe'
    import windows.locate
    WINDOWS = True

    def rename(src, dest):
        try:
            os.remove(dest)
        except:
            pass
        os.rename(src, dest)
else:
    _EXE = ''
    WINDOWS = False

    rename = os.rename

EXE_PATHS = {}


# call pipes
# see http://www.py2exe.org/index.cgi/Py2ExeSubprocessInteractions
if hasattr(sys, 'frozen') and sys.platform.startswith('win'):
    NULL_PIPE = file('nul', 'a')
    STDOUT_STDERR_PIPES = {
            'stdin': NULL_PIPE,
            'stdout': subprocess.PIPE,
            'stderr': subprocess.PIPE,
    }
    NO_PIPES = {
            'stdin': NULL_PIPE,
            'stdout': NULL_PIPE,
            'stderr': NULL_PIPE,
    }
else:
    STDOUT_STDERR_PIPES = {
            'stdout': subprocess.PIPE,
            'stderr': subprocess.PIPE,
    }
    NO_PIPES = {}

try:
    # prevent subprocesses from showing console windows
    import win32process
    STDOUT_STDERR_PIPES['creationflags'] = win32process.CREATE_NO_WINDOW
    NO_PIPES['creationflags'] = win32process.CREATE_NO_WINDOW
except ImportError:
    pass


def wrap(text, fill=70):
    return '\n'.join([textwrap.fill(line, 70)
        for line in text.split('\n')])


def title(text):
    """Turns a text in a title

    :param text: text
    :type text: str
    :returns: title
    :rtype: str

    >>> title('hello_world')
    'Hello World'
    """
    return text.replace('_', ' ').replace('-', ' ').title()


def replace_ext(filename, ext):
    return '%s.%s' % (os.path.splitext(filename)[0], ext)


#---os
def reexec_with_pythonw(py_file=None):
    """'pythonw' needs to be called for any wxPython app
    to run from the command line on Mac Os X if python version < 2.5."""
    if sys.version.split(' ')[0] < '2.5' and sys.platform == 'darwin' and\
           not (sys.executable.endswith('/Python') or hasattr(sys, 'frozen')):
        sys.stderr.write('re-executing using pythonw')
        if not py_file:
            py_file = sys.argv[0]
        os.execvp('pythonw', ['pythonw', py_file] + sys.argv[1:])


def _fix_path(path):
    if path.startswith('file://'):
        return urllib.unquote(path[7:])
    return path

if sys.platform.startswith('win'):

    def fix_path(path):
        return _fix_path(path).replace('/', os.path.sep)
else:

    def fix_path(path):
        return _fix_path(path).replace('\\', os.path.sep)


def has_ext(path, ext):
    return path.lower().endswith(ext)


def is_www_file(url):
    """Checks whether a file is remote (http or ftp).

    :param url: file path or url
    :type url: str
    :returns: True if remote, False if local
    :rtype: bool

    >>> is_www_file('http://www.foo.com/logo.png')
    True
    >>> is_www_file('ftp://foo.com/logo.png')
    True
    >>> is_www_file('logo.png')
    False
    """
    return url.startswith('http://') or url.startswith('ftp://')


def is_file(path):
    """Checks wether a path is a valid local or remote file.

    :param path: the path which has to be checked
    :type path: str
    :returns:
        True if path is a valid local or remote file, False otherwise
    :rtype: bool

    >>> is_file('http://www.foo.com/logo.png')
    True
    >>> is_file('ftp://foo.com/logo.png')
    True
    >>> is_file('/etc/fstap')
    False
    """
    return os.path.isfile(path) or is_www_file(path)


def file_extension(uri):
    return os.path.splitext(uri)[-1][1:].lower()


def filename_to_title(filename):
    """Converts a filename to a title. It replaces dashes with spaces
    and converts every first character to uppercase.

    :param filename: an absolute or relative path
    :type filename: str
    :returns: titled version of the filename
    :rtype: bool

    >>> filename_to_title('~/highlight_mask.png')
    'Highlight Mask'
    """
    return title(os.path.splitext(os.path.basename(filename))[0])


def ensure_path(path):
    """Ensure a path exists, create all not existing paths.

    It raises an OSError, if an invalid path is specified.

    :param path: the absolute folder path (not relative!)
    :type path: str
    """
    _ensure_path(os.path.abspath(path).rstrip('/').rstrip('\\'))


def _ensure_path(path):
    """Ensure a path exists, create all not existing paths.
    (Helper function for ensure_path.)"""
    if not os.path.exists(path):
        parent = os.path.dirname(path)
        if parent:
            _ensure_path(parent)
            os.mkdir(path)
        else:
            raise OSError("The path '%s' is not valid." % path)


def fix_quotes(text):
    """Fix quotes for a command line parameter. Only surround
    by quotes if a space is present in the filename.

    :param text: command line parameter
    :type text: string
    :returns: text with quotes if needed
    :rtype: string

    >>> fix_quotes('blender')
    'blender'
    >>> fix_quotes('/my programs/blender')
    '"/my programs/blender"'
    """
    if not RE_NEED_QUOTES.match(text):
        return text
    if not ('"' in text):
        return '"%s"' % text
    elif not ("'" in text):
        return "'%s'" % text
    else:
        return '"%s"' % text.replace('"', r'\"')


def set_bin_paths(paths=None):
    """Initializes where binaries can be found.

    :param paths: list of paths where binaries might be found
    :type paths: list of strings
    """
    if paths == None:
        paths = []
    global BIN_PATHS
    BIN_PATHS = paths


def find_in(filename, paths):
    """Finds a filename in a list of paths.

    :param filename: filename
    :type filename: str
    :param paths: paths
    :type paths: list of strings
    :returns: found filename with path or None
    :rtype: string or None
    """
    for path in paths:
        p = os.path.join(path, filename)
        if os.path.isfile(p):
            return p
    return None


def find_exe(executable, use_which=True, raise_exception=False):
    """Finds an executable binary. Returns None if the binary can
    not be found.

    This method need some extra love for Windows and Mac.

    :param executable: binary which will be used as a plugin (eg imagemagick)
    :type executable: string
    :param quote: quote the path if it contains spaces
    :type quote: bool
    :param use_which: use the command ``which`` on non windows platforms
    :type use_which: bool
    :param raise_exception: raise exception if not found
    :type raise_exception: bool
    :returns: absolute path to the binary
    :rtype: string or None
    """
    try:
        return EXE_PATHS[executable]
    except KeyError:
        pass
    #executable with extension e.g. exe on windows
    executable_exe = executable
    if not executable_exe.endswith(_EXE):
        executable_exe += _EXE
    # try to first find in BIN_PATHS
    executable_path = find_in(executable_exe, BIN_PATHS)
    # try to find a system install (todo for windows)
    if not sys.platform.startswith('win') and use_which and \
            executable_path is None:
        path, stderr, err = call_out_err(['which', executable_exe])
        if not err:
            executable_path = path.strip()
    if executable_path is None:
        executable_path = find_in(executable_exe,
            os.environ['PATH'].split(os.pathsep))
        if (executable_path is None) and WINDOWS:
            executable_path = windows.locate.find_exe(executable)
    #cache and return the result
    EXE_PATHS[executable] = executable_path
    if executable_path is None and raise_exception:
        raise IOError('No such program: %s' % executable)
    return executable_path


def find_command(text):
    """Find command in text

    :param text: command line
    :type text: string
    :returns: text
    :rtype: text

    >>> find_command('convert image.jpg image.jpg')
    'convert'
    >>> find_command('"/my apps/convert" image.jpg image.jpg')
    '"/my apps/convert"'
    >>> find_command('/my apps/convert image.jpg image.jpg')
    '/my'
    """
    match = RE_COMMAND.match(text)
    if match:
        return match.group(0)
    return None


class TempFile:

    def __init__(self, suffix='', path=None):
        """Make a temporary file with :func:`tempfile.mkstemp`. Use
        the ``path`` attribute to get the filename.

        :param suffix:

            If suffix is specified, the file name will end with that
            suffix, otherwise there will be no suffix. ``TempFile`` does
            not put a dot between the file name and the suffix; if you
            need one, put it at the beginning of suffix.

        :type suffix: string
        :param path: prefabricated temp path
        :type path: string

        >>> t = TempFile('.png')
        >>> t.path.endswith('.png')
        True
        >>> t.close()
        """
        if path is None:
            self._fd, self.path = tempfile.mkstemp(suffix)
        else:
            self._fd = None
            self.path = path
        self._closed = False

    def close(self, force_remove=True, dest=''):
        """It is important to call this method when finished with
        the temporary file.

        :param force_remove:

            Remove temporary file and raise IOError when it does not exist
            anymore. Set to False to allow for processes that:
            - delete the temporary files themselves
            - might have failed to produce an output file

        :type force_remove: boolean
        :param dest:

            This is eg used for thumbnails in order to rename them to
            their proper location

        :type dest: string

        """
        if self._closed:
            raise IOError("Temporary file '%s' is already closed." % self.path)
        if self._fd:
            os.close(self._fd)
            self._fd = None
        if not (force_remove or os.path.exists(self.path)):
            self._closed = True
            return
        if dest:
            if os.path.exists(dest):
                os.remove(dest)
            shutil.move(self.path, dest)
        else:
            os.remove(self.path)
        self._closed = True


def _try_also_encoded(method, args, **options):
    try:
        return method(args, **options)
    except UnicodeEncodeError:
        pass
    # mostly needed on Windows, see:
    # - http://bugs.python.org/issue1759845
    # - http://bugs.python.org/file11674/Python-2.5.2-subprocess.patch
    if not isinstance(args, types.StringTypes):
        args = subprocess.list2cmdline(args)
    if isinstance(args, unicode):
        args = args.encode(FS_ENCODING)
    return method(args, **options)


def call(args, **options):
    """Runs a shell command and returns it's exit code. This also is an
    obligatory alternative to subprocess.call as that won't work with
    py2exe.

    :param args: the command to be executed in the shell
    :type args: tuple of strings
    :returns: command exit code
    :rtype: integer
    """
    options.update(NO_PIPES)
    pipe = _try_also_encoded(subprocess.Popen, args, **options)
    pipe.communicate()
    return pipe.returncode


def call_out_err(args, **options):
    """Runs a shell command and captures the output.

    :param args: the command to be executed in the shell
    :type args: tuple of strings
    :returns: stdout, stderr and return code
    :rtype: tuple of (str, str, int)

    The options will be passed to the subprocess.Popen

    >>> call_out_err(['echo', 'world'])
    ('world\\n', '', 0)
    """
    options.update(STDOUT_STDERR_PIPES)
    pipe = _try_also_encoded(subprocess.Popen, args, **options)
    stdout_data, stderr_data = pipe.communicate()
    return stdout_data, stderr_data, pipe.returncode


def call_out_err_temp(args, input=None, output=None, needs_temp=None,
        output_ext=None, **options):
    """Same function as :func:`call_out_err` but hacks around problematic
    unicode filenames with temp files. This will probably only be needed
    by certain Windows programs.

    We can't use hard links as on windows that only works for NTFS.

    :param args: the command to be executed in the shell
    :type args: tuple of strings
    :param input: input files which could be converted to temp files
    :type input: list of str or unicode
    :param output: output files which could be converted to temp files
    :type output: list of str or unicode
    :param needs_temp:

        function(stderr, stdout, err) which evaluates if it is worth to convert
        to temp files or not

    :type needs_temp: function
    :param output_ext:

        extension of generated output file which is not listed in the command
        (e.g. necessary for dcraw)

    :type output_ext: None
    :returns: stdout, stderr and return code
    :rtype: tuple of (str, str, int)
    """
    stdout, stderr, err = call_out_err(args, **options)
    if not(err and (needs_temp is None or needs_temp(stdout, stderr, err))):
        return stdout, stderr, err
    if input is None:
        input = []
    if output is None:
        output = []
    temp_input = []
    temp_output = []
    temp_args = []
    try:
        for arg in args:
            if arg in input + output:
                temp_file = TempFile()
                temp_args.append(temp_file.path)
                if arg in input:
                    shutil.copy2(arg, temp_file.path)
                    temp_input.append((temp_file, arg))
                else:
                    temp_output.append((temp_file, arg))
            else:
                temp_args.append(arg)
        stdout, stderr, err = call_out_err(temp_args, **options)
        return stdout, stderr, err
    finally:
        for temp_file, source in temp_input:
            temp_file.close()
            if output_ext:
                shutil.move(
                    replace_ext(temp_file.path, output_ext),
                    replace_ext(source, output_ext),
                )
        for temp_file, dest in temp_output:
            temp_file.close(dest=dest, force_remove=False)


def shell_cache(args, cache='', key=None, validate=None, **options):
    """Runs a shell command and captures the output. It uses a caching system
    so that cached results don't need to run a subprocess anymore. The results
    are cached by sys.platform

    :param args: the command to be executed in the shell
    :type args: tuple of strings
    :param cache: the filename of the cache file
    :type cache: string
    :param key:
        normally the result is stored in as cache_dict[args], but this can
        overwritten by
    :param validate: a validate (eg mtime) to validate the cache result
    :returns: stdout and stdout
    :rtype: typle of strings
    """
    # Initialize cache_dict
    args = tuple(args)
    cache_dict = {}
    if key is None:
        key = args
    # Try to load from cache file
    if os.path.isfile(cache):
        f = open(cache, 'rb')
        source = f.read()
        f.close()
        try:
            cache_dict = safe.eval_safe(source)
        except SyntaxError:
            pass
    # Initialize result
    result = None
    # Is it cached already?
    if key in cache_dict:
        x = cache_dict[key]
        if sys.platform in x:
            x = x[sys.platform]
            if validate:
                if validate == x['validate']:
                    result = x
            else:
                result = x
    if result is None:
        # Add to cache
        result = {'validate': validate}
        result['stdout'], result['stderr'], result['err'] = \
              call_out_err(args, **options)
        if not key in cache_dict:
            cache_dict[key] = {}
        cache_dict[key][sys.platform] = result
        # Save to cache
        ensure_path(os.path.dirname(cache))
        f = open(cache, 'wb')
        f.write(unicode(cache_dict))
        f.close()
    return result['stdout'], result['stderr'], result['err']


def start(path):
    """Open a file or browse a folder.

    :param path: location of the file
    :type path: string
    """
    if hasattr(os, 'startfile'):
        #windows
        os.startfile(path)
    else:
        if sys.platform.startswith('darwin'):
            #mac
            command = 'open'
        else:
            #linux
            command = 'xdg-open'
        call('%s "%s"' % (command, path), shell=True)


class MethodRegister:

    def __init__(self):
        """Creates a register where methods to open files are registered
        by the extensions.

        >>> m = MethodRegister()
        >>> m.register(['bmp','png'], abs)
        >>> m.register(['png'], open)
        >>> m.register(['bmp'], None)
        >>> 'bmp' in m.extensions
        True
        >>> m.get_methods('bmp')
        [<built-in function abs>]
        >>> m.get_methods('png')
        [<built-in function abs>, <built-in function open>]
        >>> m.unregister_method(abs)
        >>> m.get_methods('bmp')
        []
        >>> 'bmp' in m.extensions
        False
        >>> m.get_methods('png')
        [<built-in function open>]
        """
        self._dict = {}
        self._methods = {}
        self._extensions = {}
        self.extensions = []

    def register(self, extensions, method):
        """Register one method for multiple extensions. If the method
        is None, it will cancel the registration.

        :param extensions: list of file extensions
        :type extensions: list of strings
        :param method: method to open a file
        :param method: function
        """
        if method is None:
            return
        for extension in extensions:
            if not(extension in self._methods):
                self._methods[extension] = []
            self._methods[extension].append(method)
        if not (method in self._extensions):
            self._extensions[method] = []
        self._extensions[method].extend(extensions)
        self._update()

    def does_process(self, filename):
        """Check if the filename can be processed by any of the
        registered methods.

        :param filename: filename
        :type filename: string
        """
        return file_extension(filename) in self.extensions

    def get_methods(self, extension):
        """Get all methods registered for an extension.

        :param extension: file extension
        :type extension: string
        :returns: list of methods registered for the extension
        :rtype: list
        """
        return self._methods.get(extension, [])

    def unregister_method(self, method):
        """Unregister a method from all extensions.

        :param method: method to open a file
        :param method: function
        """
        self._unregister(method, self._extensions, self._methods)

    def unregister_extension(self, extension):
        """Unregister an extension from all methods.

        :param extensions: list of file extensions
        :type extensions: list of strings
        """
        self._unregister(extension, self._methods, self._extensions)

    def _unregister(self, key, d, values):
        """Helper method for :ref:`unregister_method` and
        :ref:`unregister_extension`.

        :param key: method/extension
        :param d: self._extensions/self._methods
        :type d: dict
        :param values: self._methods/self._extensions
        :type values: dict
        """
        if key in d:
            for value in values.keys():
                values[value] = [x for x in values[value] if x != key]
                if not values[value]:
                    del values[value]
            del d[key]
            self._update()

    def _update(self):
        """Updates the list of extension after each change. Helper
        function for :ref:`register` and :ref:`_unregister`."""
        self.extensions = self._methods.keys()
