# -*- coding: utf-8 -*-
# Phatch - Photo Batch Processor
# Copyright (C) 2009 Nadia Alramli, Stani (www.stani.be)
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/
#
# Phatch recommends SPE (http://pythonide.stani.be) for editing python files.
#
# Follows PEP8

import os.path
import re
import shutil

from jinja2 import Environment, PackageLoader

import cgi
import codecs
from collections import defaultdict
import config
import fnmatch
import utils
import pprint

env = Environment(
    loader=PackageLoader('test_suite', 'templates')
)

ERROR_REG = re.compile(
    r'Image:\s+(?P<image>[\w/-_.]+).+Details:\s+(?P<error>.+)',
    re.MULTILINE | re.DOTALL | re.IGNORECASE | re.UNICODE
)


class Column(object):
    def __init__(self, value, cls=None, url=None):
        self.value = value
        self.cls = cls
        self.url = url


class Row(object):
    def __init__(self, columns, cls=None):
        self.columns = columns
        self.cls = cls

    def __repr__(self):
        return repr(self.columns)


class Table(object):
    def __init__(self, header):
        self.header = header
        self.rows = []

    def add_row(self, row, url=None, cls=None):
        columns = [
            Column(column, head.lower().replace(' ', '_'), url)
            for head, column in zip(self.header, row)
        ]
        self.rows.append(Row(columns, cls))


def clean():
    if os.path.exists(config.HTML_REPORT):
        shutil.rmtree(config.HTML_REPORT)


def parse_log(path):
    report = defaultdict(list)
    logs = codecs.open(path, encoding='utf-8').read()
    logs = logs.split('#####' * 10)
    for log in logs:
        log = log.strip(' \n\r')
        if not log:
            break
        lines = log.splitlines()
        actionlist = lines[0]
        if len(lines) <= 1:
            errors = []
        else:
            dummy, errors = log.split('\n', 1)
            errors = errors.split('\n\n\n')
            errors = [parse_error(error) for error in errors]

        fields = os.path.basename(actionlist)[:-7].split('-')
        actions = ', '.join(
            ' '.join(action.split('_'))
            for action in fields[0].split('__')
        )
        actions = actions
        if not actions.istitle():
            actions = actions.title()

        input = dict(
            (
                os.path.splitext(path)[0],
                os.path.join(config.DEFAULT_INPUT, path)
            )
            for path in os.listdir(unicode(config.DEFAULT_INPUT))
        )

        images, original = find_output(actionlist, input)
        report[actions].append({
            'name': os.path.basename(actionlist)[:-7],
            'fields': fields,
            'errors': errors,
            'actionlist': actionlist,
            'output': images,
            'original': original,
            'input': input,
        })
    return report


def index_page(report):
    table = Table(['Action', 'Tests', 'Errors', 'Output', 'Missing', 'Notice'])
    for action, tests in sorted(report.iteritems()):
        num_errors = num_output = num_tests = num_input = 0
        is_critical = False
        for test in tests:
            num_errors += len(test['errors'])
            num_output += len(test['output'])
            num_tests += 1
            num_input += len(test['input'])
            if [
                    error['image']
                    for error in test['errors']
                    if error['image'] is None
                ]:
                is_critical = True
        if num_errors:
            if is_critical:
                cls = 'critical'
            else:
                cls = 'warn'
        elif num_input > num_output:
            cls = 'note'
        else:
            cls = None
        table.add_row(
            [
                action,
                num_tests,
                num_errors,
                num_output,
                num_input - num_output,
                cls.title() if cls else '',
            ],
            url='%s.html' % action.lower().replace(' ', '_'),
            cls=cls
        )
    template = env.get_template('index.html')
    f = open(os.path.join(config.HTML_REPORT, 'index.html'), 'w')
    f.write(template.render(table=table).encode('utf-8'))
    f.close()


def find_output(actionlist, input):
    images = []
    name = os.path.basename(actionlist)[:-7]
    pattern = '*-%s.*' % name
    original = []
    for image in os.listdir(unicode(config.DEFAULT_OUTPUT)):
        if fnmatch.fnmatch(image, pattern):
            images.append(
                os.path.join(config.DEFAULT_OUTPUT, image)
            )
            name, dummy = image.split('-', 1)
            original.append(input[name])
    return sorted(images), sorted(original)


def action_page(action, tests):
    table = Table(['Fields', 'Errors', 'Output', 'Notice', 'Action List'])
    for test in tests:
        test_page(test)
        images = ''.join(
            '<img src="%s" height="128px" alt="Cannot Open"/>' % img
            for img in test['output']
        )
        url = '%s-test.html' % test['name']
        if not images:
            images = 'No output'
        else:
            images = '<a href="%s">%s</a>' % (url, images)
        if test['errors']:
            if [
                    error['image']
                    for error in test['errors']
                    if error['image'] is None
                ]:
                cls = 'critical'
            else:
                cls = 'warn'
        elif len(test['input']) > len(test['output']):
            cls = 'note'
        else:
            cls = None
        warn = '<span class="critical">Critical</span>'
        errors = '<pre>%s</pre>' % '<hr/>'.join(
            '%s<br>%s' % (
                os.path.basename(error['image']) if error['image'] else warn,
                cgi.escape(error['error']),
            )
            for error in test['errors']
        )
        table.add_row(
            [
                '<br/>'.join(
                    map(cgi.escape, test['fields'])
                ).replace('=', ' = '),
                errors,
                images,
                cls.title() if cls else '',
                '<a href="%s">Download</a>' % test['actionlist'],
            ],
            url='',
            cls=cls,
        )
    template = env.get_template('action.html')
    path = '%s.html' % action.lower().replace(' ', '_')
    f = open(os.path.join(config.HTML_REPORT, path), 'w')
    f.write(template.render(table=table).encode('utf-8'))
    f.close()


def parse_error(error):
    match = ERROR_REG.search(error)
    if not match:
        return {'image': None, 'error': error}
    return match.groupdict()


def test_page(test):
    table = Table(['Fields', 'Error', 'Original', 'Output'])
    for orig, out in zip(test['original'], test['output']):
        error = '<pre>%s</pre>' % '<hr/>'.join(
            '%s<br>%s' % (
                os.path.basename(error['image']) if error['image'] else None,
                cgi.escape(error['error']),
            )
            for error in test['errors'] if error['image'] == orig
        )
        table.add_row([
            '<br/>'.join(
                map(cgi.escape, test['fields'])
            ).replace('=', ' = '),
            error,
            '<img src="%s" width="256" />' % orig,
            '<img src="%s" width="256" />' % out,
        ])
    template = env.get_template('action.html')
    path = '%s-test.html' % test['name']
    f = open(os.path.join(config.HTML_REPORT, path), 'w')
    f.write(template.render(table=table).encode('utf-8'))
    f.close()


def run():
    clean()
    report = parse_log(config.DEFAULT_LOG)
    shutil.copytree(config.HTML_TEMPLATES, config.HTML_REPORT)
    index_page(report)
    for action, tests in sorted(report.iteritems()):
        action_page(action, tests)
